// ==UserScript==
// @name        HNhance
// @version     1.4
// @description Adds extra functions to Hacker News
// @match       http://news.ycombinator.com/*
// @match       https://news.ycombinator.com/*
// ==/UserScript==

/* See the README for abstract + license information! */

(function() {
	"use strict";
	
	// Helpers
	// ```````
	
	var injectStyleFrom = function(path) {
		$(function() {
			$("head").append($("<link>").attr({
				'rel': 'stylesheet',
				'type': 'text/css',
				'href': window.chrome.extension.getURL(path)
			}));
		});
	};
	
	var settings_get = function() {
		try {
			return JSON.parse(localStorage["hnhance"]);
		} catch(em_all) {
			return {
				'features': {
					'styleCommentLink': false,
					'hoverParents': true,
					'show4chan': true,
					'customStyle': true,
					'showSubmissionsLink': true,
					'confirmDownvotes': false,
					'collapseComments': false,
					'hoverProfiles': true,
					'showSubmitterPosts': true
				},
				'theme': 'modern'
			};
		}
	};
	
	var settings_save = function(o) {
		localStorage["hnhance"] = JSON.stringify(o);
	};
	
	var settings_enum = {
		'styleCommentLink': 'Dim visited comment links',
		'hoverParents': 'Show parent hover links',
		'show4chan': 'Show related 4chan /g/ links',
		'showSubmissionsLink': 'Show submissions link',
		'confirmDownvotes': 'Confirm downvotes',
		'collapseComments': 'Allow collapsing threads',
		'hoverProfiles': 'Display user info on hover',
		'showSubmitterPosts': 'Highlight comments from submitter'
	};
	
	var settings_themes_enum = {
		'default': 'No custom theme',
		'segoe': 'HN-Segoe',
		'modern': 'Modern',
		'georgify': 'GeorgifyMod',
		'georgify-monokai': 'Georgify Monokai'
	};
	
	/*!
	 * Feature: settingsDialog
	 * Always enabled.
	 */	
	window.addEventListener("load", function() {
		
		injectStyleFrom("hnhance.css");
		
		var selm = document.createElement("a");
		selm.id = "hnh-settings";
		selm.onclick = function() {
			document.getElementById("hnh-settings-dialog").style.display = (document.getElementById("hnh-settings-dialog").style.display == "block") ? "none" : "block";
		};
		
		var t = document.getElementsByClassName("pagetop")[1];
		t.insertAdjacentHTML("afterbegin", " | ");
		t.insertAdjacentElement("afterbegin", selm);
		
		var d = document.createElement("div");
		d.id="hnh-settings-dialog";
		d.className="hnh-dialog";
		d.style.display = "none";
		d.innerHTML = '<strong>HNHance options</strong><br><em><a href="javascript:;" onclick="window.location.reload();">Reload</a> to see changes.</em><br>';
				
		for (var i in settings_enum) {
			var lb = document.createElement("label");
			var cb = document.createElement("input");
			cb.type = "checkbox";
			cb.onclick = (function(i) {
				return function() {
					var s = settings_get();
					s.features[i] = this.checked;
					settings_save(s);
				}
			}(i));
			
			if (settings_get().features[i]) cb.checked = 'checked';
			lb.appendChild(cb);
			lb.appendChild(document.createTextNode(settings_enum[i]));
			d.appendChild(lb);
			d.appendChild(document.createElement('br'));
		}
		
		d.insertAdjacentHTML("beforeend", "<strong>Custom themes</strong><br>");
		
		for (var i in settings_themes_enum) {
			var lb = document.createElement("label");
			var rb = document.createElement("input");
			rb.type = "radio";
			rb.name = "hnhance_custom_theme";
			rb.onclick = (function(i) {
				return function() {
					var s = settings_get();
					s.theme = i;
					settings_save(s);
				}
			}(i));
			
			if (settings_get().theme == i) rb.checked = 'checked';
			lb.appendChild(rb);
			lb.appendChild(document.createTextNode(settings_themes_enum[i]));
			d.appendChild(lb);
			d.appendChild(document.createElement('br'));
		}
		
		document.body.appendChild(d);
		
	}, false);
	
	/*!
	 * Feature: styleCommentLink
	 * Adds a visible a:visited color for comment links.
	 *
	 * @author mappy
	 * @version 0.1~hnhance1
	 */
	if (settings_get().features.styleCommentLink) $(function() {	
		$("body").addClass('hnhance-style-comment-links');
	});
	
	/*!
	 * Feature: showSubmissionsLink
	 * Adds an extra item to the top bar
	 *
	 * @author mappy
	 * @version 0.1~hnhance1
	 */
	if (settings_get().features.showSubmissionsLink) $(function() {
		var threadsLink = document.querySelectorAll(".pageTop a")[2];
		var unc = threadsLink.href.indexOf("threads");
		if (unc == -1) {
			// not logged in
		} else {
			var username = threadsLink.href.substr(unc + 11);
			threadsLink.insertAdjacentHTML(
				"afterend",
				" | <a href='"+document.location.protocol+"//news.ycombinator.com/submitted?id="+username+"'>submissions</a>"
			);
		}
	});
	
	/*!
	 * Emphasise comments made by the submitter.
	 *
	 * @author mappu
	 */
	if (settings_get().features.showSubmitterPosts) $(function() {
		
		var $a = $(".subtext a[href^=user]");
		if ($a.length === 1) {
			var username = $a.text();
			
			console.log('submitter is '+username);
			
			$("a[href^=user]").each(function() {
				if ($(this).text() == username) {
					$(this).addClass('hnhance-is-article-submitter');
				}
			});
		}
		
	});
	
	/*!
	 * Feature: confirmDownvotes
	 * Displays a confirmation dialog in case you accidentally fat-fingered the
	 *  downvote button.
	 *
	 * @author mappy
	 * @version 0.0~hnhance1
	 */
	if (settings_get().features.confirmDownvotes) $(function() {
		$(".rotate180").closest("a").each(function() {
			var lasthref = $(this).attr('href');
			$(this).removeAttr('onclick').off("click");
			$(this).attr('data-oldhref', $(this).attr('href'));
			$(this).attr('href', 'javascript:;');
			
			$(this).click(function(ev) {
				if (confirm("Are you sure you want to downvote this post?")) {
					$(this).attr('href', $(this).attr('data-oldhref'));
					vote( $(this)[0] );
				}
				ev.preventDefault();
				return false;
			});
		});
	});
	
	/*!
	 * Feature: hoverParents
	 * Displays a mouseover link to view the parent comment.
	 *
	 * @author mappy
	 * @version 1.0~hnhance1
	 */	 
	if (settings_get().features.hoverParents) $(function() {
		
		// Determine chain of parents
		
		var rows = document.getElementsByTagName("table")[3].children[0].children;
		var last = {};
		for (var i in rows) {
			if (! rows[i].children) continue;
		
			try {
				rows[i].id = "hnpost_"+i;
				
				var width = rows[i].
					children[0].children[0].children[0].
					children[0].children[0].children[0].
					width;
				
				last[width] = i;
				
				if ((width-40) in last) {
					rows[i].setAttribute("data-hnparent", last[(width-40)]);
				}
			} catch (em_all) {}
		}
		last = {};
			
		// Create hover element
		
		var hover = document.createElement("div");
		//hover.id            = 'hnhance_tooltip';
		hover.style.display = 'none';
		hover.className     = "hnh-hover";
		document.body.appendChild(hover);
		
		hover.appendChild(document.createTextNode("loading..."));
		
		// Create parent links
		
		var u_tags = document.getElementsByTagName("u");
		for (var i = 0; i < u_tags.length; i++) {
			// Walk backward to <tr>. Root posts are one level higher
			var tr = $(u_tags[i]).closest("tr").parent().closest("tr")[0];
			
			var post_id = (tr.id) ? tr.id : tr.parentNode.id;
			
			if (! (""+post_id).length) {
				continue;
			}
			
			// Find parent post id
			var parent_id = document.getElementById(post_id).
				getAttribute("data-hnparent");
			
			if (! parent_id) continue;
			
			// Create parent link:
			var parent_link = document.createElement("a");
			parent_link.id = "hnparentlink_"+i;
			parent_link.className="hnh-parentlink";
			parent_link.setAttribute("href", "javascript:;");
			parent_link.appendChild(document.createTextNode("parent"));
			
			// Mouseover
			var mouseover = function(post_id, e) {
				while (hover.hasChildNodes()) {
					hover.removeChild(hover.lastChild);
				}
				// Retrieve
				var src = document.getElementById(post_id).
					children[0].children[0].children[0].
					children[0].children[2].children;
				for (var i = 0; i < src.length; i++) {
					hover.appendChild(src[i].cloneNode(true));
				}
				hover.style.display = 'block';
			};
			parent_link.onmouseover = mouseover.bind(
				mouseover, "hnpost_"+parent_id
			);
			
			// Mousemove
			parent_link.onmousemove = function(e) {
				hover.style.top = e.clientY + 10;
				hover.style.left = e.clientX + 10;
			}
			
			// Mouseout
			parent_link.onmouseout = function() {
				hover.style.display = 'none';
			};
			
			// Add to document
			u_tags[i].parentNode.appendChild(document.createTextNode(" | "));
			var u_el = document.createElement("u");
			//u_el.appendChild(parent_link);
			//u_tags[i].parentNode.appendChild(u_el);
			u_tags[i].parentNode.appendChild(parent_link);
		}
		
	});
	
	/*!
	 * Feature: hoverProfiles
	 * Mouse over a user's name to view their profile stats.
	 *
	 * @author mappu
	 * @version 2.0~hnhance1
	 */
	if (settings_get().features.hoverProfiles) $(function() {
	
		var h_el = document.createElement("div");
		h_el.style.display = 'none';
		h_el.className     = "hnh-hover";
		document.body.appendChild(h_el);
		
		var state = {};
		
		var a_els = document.getElementsByTagName("a");
		for (var i in a_els) {
			if (! a_els[i].href) {
				continue;
			}
			
			var upos = a_els[i].href.indexOf("user?");
			if (upos === -1) {
				continue;
			}
			
			state[ a_els[i].href ] = [ 0, null ]; // 0: no hover. 1: in delay. 2: in GET. 3: result cached
			
			a_els[i].addEventListener("mouseover", (function(url) {
				return function(e) {
					
					h_el.style.top = e.clientY + 10;
					h_el.style.left = e.clientX + 10;
					h_el.style.display = "block";
					h_el.innerHTML = "Loading...";
					
					if (state[url][0] == 3) {
						h_el.innerHTML = state[url][1];
						
					} else {
						var timer = setTimeout(function() {
							state[url] = [2, null];
							
							$.get(url, function(ret) {
								state[url] = [3, (""+ret).replace(/[\r\n]/g, " ").match(/<table border="0">(.*?)\/table>/g, "$1") ];
								h_el.innerHTML = state[url][1];
							});
							
						}, 300);
						
						state[url] = [1, timer];
						
					}
					
				};
			}(a_els[i].href)));
			
			a_els[i].addEventListener("mousemove", function(e) {
				h_el.style.top = e.clientY + 10;
				h_el.style.left = e.clientX + 10;
			});
			
			a_els[i].addEventListener("mouseout", function() {
				var url = this.href;
				
				if (state[url][0] == 1) {
					clearTimeout(state[url][1]);
					state[url] = [0, null];
					
				}
				h_el.style.display = "none";
			});
			
		}
	});
	
	/*!
	 * Feature: hn4chan
	 * Display 4chan/g/ related threads on the homepage
	 *
	 * @author  anon
	 * @grant   none
	 * @version 1.0.0~hnhance2
	 */
	if (settings_get().features.show4chan) $(function() {
		
		// No point loading this mod on comment pages
		if (document.location.href.indexOf("/item?") !== -1) return;
		
		var xhr = new XMLHttpRequest();
		xhr.open("GET", document.location.protocol + "//api.4chan.org/g/catalog.json");

		xhr.onload = function() {
			var catalog = JSON.parse(xhr.responseText);

			[].slice.call(document.querySelectorAll("td.subtext > a[href^=item]"))
				.map(function(commentLink) {
					var itemLink = commentLink
						.parentNode.parentNode.previousSibling.querySelector("td.title > a");

					return {
						link:
							itemLink
								.getAttribute("href")
								.replace(/^http:\/\/(?:www.)?|^https:\/\/(?:www.)?|\/$/g, ""),
						comment:
							"news.ycombinator.com/" +
							commentLink.getAttribute("href"),
						elem: commentLink.parentNode,
					};
				})
				.forEach(function(item) {
					catalog.forEach(function(page) {
						page.threads
							.filter(function(thread) { return thread.com; })
							.forEach(function(thread) {
								var comment = thread.com
									.replace(/<wbr>|<br>/g, "")
									.replace(/&lt;/g, "<")
									.replace(/&gt;/g, ">")
									.replace(/&#039;/g, "'")
									.replace(/&quot;/g, "\"")
									.replace(/&amp;/g, "&");

								if (
									comment.indexOf(item.link) != -1 ||
									comment.indexOf(item.comment) != -1)
								{
									var a = document.createElement("a");
									a.setAttribute(
										"href",
										document.location.protocol + "//boards.4chan.org/g/res/" + thread.no
									);
									a.appendChild(document.createTextNode("4chan"));

									item.elem.appendChild(document.createTextNode(" | "));
									item.elem.appendChild(a);
								}
							});
					});
				});
		};

		xhr.send();

	});
	
	/*!
	 * Feature: collapseComments
	 * Provides a little squared + or squared - icon which can be clicked to
	 *   hide a tree of comments in a main Hacker News thread.
	 *
	 * @ref https://github.com/niyazpk/Collapsible-comments-for-Hacker-News
	 * @author niyazpk
	 * @version fdeddab1ec~hnhance1
	 */
	if (settings_get().features.collapseComments) $(function() {
		
		var current_level_width = 0;
		var inner_level_width = 1000;
    
		if (!$('body').hasClass('collapsible-comments')) {
			
			$('body').addClass('collapsible-comments');
			var span_html = '<span style="cursor:pointer;" class="hnhance-expand expand-handle">[-]</span>';
			
			if (window.location.href.indexOf('item?id=') != -1) {
			    if ($('center > table > tbody > tr').length > 4) {
				$('center > table > tbody > tr:eq(3) > td > table:eq(1) span.comhead').prepend(span_html);
			    } else {
				$('center > table > tbody > tr:eq(2) > td > table:eq(1) span.comhead').prepend(span_html);
			    }
			} else if (window.location.href.indexOf('threads?id=') != -1) {
			    $('center > table > tbody > tr > td > table span.comhead').prepend(span_html);
			}

			var collapse = function() {

			    current_level_width = parseInt($(this).closest('tr').find('td:eq(0) > img').attr('width'), 10);

			    $(this).closest('table').closest('tr').nextAll().each(function(index, el) {
				var elWidth = parseInt($('tbody > tr > td > img', this).attr('width'), 10);
				console.log(elWidth > current_level_width, typeof elWidth);
				if (elWidth > current_level_width) {
				    if (elWidth <= inner_level_width) {
					inner_level_width = 1000;
					$(this).hide();
				    }
				    if (inner_level_width == 1000 && $('.comment', this).css('display') == 'none') {
					inner_level_width = elWidth;
				    }
				} else {
				    return false;
				}
			    });

			    inner_level_width = 1000;
			    $(this).text('[+]').addClass('expand-handle-collapsed').removeClass('expand-handle');
			    $(this).closest('div').nextAll().hide();
			    $(this).closest('div').parent().prev().hide();
			    $(this).closest('div').css({
				'margin-left': '18px',
				'margin-bottom': '5px'
			    });
			};
			
			var uncollapse = function() {

			    current_level_width = parseInt($(this).closest('tr').find('td > img').attr('width'), 10);

			    $(this).closest('table').closest('tr').nextAll().each(function(index, el) {
				var elWidth = parseInt($('tbody > tr > td > img', this).attr('width'), 10);
				if (elWidth > current_level_width) {
				    if (elWidth <= inner_level_width) {
					inner_level_width = 1000;
					$(this).show();
				    }
				    if (inner_level_width == 1000 && $('.comment', this).css('display') == 'none') {
					inner_level_width = elWidth;
				    }
				} else {
				    return false;
				}
			    });

			    inner_level_width = 1000;
			    $(this).text('[-]').addClass('expand-handle').removeClass('expand-handle-collapsed');
			    $(this).closest('div').nextAll().show();
			    $(this).closest('div').parent().prev().show();
			    $(this).closest('div').css({
				'margin-left': '0',
				'margin-bottom': '-10px'
			    });
			};
			
			$('.hnhance-expand').click(function() {
				if ($(this).is(".expand-handle-collapsed")) {
					uncollapse.apply(this);
				} else {
					collapse.apply(this);
				}
			});
		}
	});
	
	//
	// THEMES
	//
	
	switch(settings_get().theme) {
		
		case 'segoe':
		case 'modern':		
		case 'georgify':		
		case 'georgify-monokai':
		case 'default':
			injectStyleFrom("theme-" + settings_get().theme + ".css");
		break;
		
		default:
			injectStyleFrom("theme-default.css");
		break;
	}
})();
